#!/bin/ksh
#dis
#dis    Open Source License/Disclaimer, Forecast Systems Laboratory
#dis    NOAA/OAR/FSL, 325 Broadway Boulder, CO 80305
#dis
#dis    This software is distributed under the Open Source Definition,
#dis    which may be found at http://www.opensource.org/osd.html.
#dis
#dis    In particular, redistribution and use in source and binary forms,
#dis    with or without modification, are permitted provided that the
#dis    following conditions are met:
#dis
#dis    - Redistributions of source code must retain this notice, this
#dis    list of conditions and the following disclaimer.
#dis
#dis    - Redistributions in binary form must provide access to this
#dis    notice, this list of conditions and the following disclaimer, and
#dis    the underlying source code.
#dis
#dis    - All modifications to this software must be clearly documented,
#dis    and are solely the responsibility of the agent making the
#dis    modifications.
#dis
#dis    - If significant modifications or enhancements are made to this
#dis    software, the FSL Software Policy Manager
#dis    (softwaremgr@fsl.noaa.gov) should be notified.
#dis
#dis    THIS SOFTWARE AND ITS DOCUMENTATION ARE IN THE PUBLIC DOMAIN
#dis    AND ARE FURNISHED "AS IS."  THE AUTHORS, THE UNITED STATES
#dis    GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND
#dis    AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS
#dis    OF THE SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE.  THEY ASSUME
#dis    NO RESPONSIBILITY (1) FOR THE USE OF THE SOFTWARE AND
#dis    DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT TO USERS.
#dis
#dis

##########################################################################
#
#Script Name: bucket.ksh
# 
#     Author: Christopher Harrop
#             Forecast Systems Laboratory
#             325 Broadway R/FST  
#             Boulder, CO. 80305
#
#   Released: 10/30/2003
#    Version: 1.0
#    Changes: None
#
# Purpose: This script bucket processes wrf output.  It is based on scripts
#          whose authors are unknown.
#
#               EXE_ROOT = The full path of the bucket executables
#          MOAD_DATAROOT = Top level directory of wrf output and
#                          configuration data.
#             START_TIME = The cycle time to use for the initial time. 
#                          If not set, the system clock is used.
#              FCST_TIME = The two-digit forecast that is to be bucketed
# 
# A short and simple "control" script could be written to call this script
# or to submit this  script to a batch queueing  system.  Such a "control" 
# script  could  also  be  used to  set the above environment variables as 
# appropriate  for  a  particular experiment.  Batch  queueing options can
# be  specified on the command  line or  as directives at  the top of this
# script.  A set of default batch queueing directives is provided.
#
##########################################################################

# Set the SGE queueing options 
#$ -S /bin/ksh
#$ -pe ncomp 1
#$ -l h_rt=6:00:00
#$ -N bucket_test
#$ -j y
#$ -V

# Set up paths to shell commands
LS=/bin/ls
LN=/bin/ln
RM=/bin/rm
MKDIR=/bin/mkdir
CP=/bin/cp
MV=/bin/mv
ECHO=/bin/echo
CAT=/bin/cat
GREP=/bin/grep
CUT=/bin/cut
AWK="/bin/gawk --posix"
SED=/bin/sed
DATE=/bin/date
BC=/usr/bin/bc
MPIRUN=mpirun

export WRF_ROOT=/p60/wrf-dte/WRF/unified/WRFV2
export EXE_ROOT=/p60/wrf-dte/wrfpostproc/exec
export MOAD_DATAROOT=/p60/wrf-dte/wrfpostproc/conus13_nmm
#export MOAD_DATAROOT=/p60/wrf-dte/WRFRR/DOMAINS/NMM/conus13/ph1_NAM/2006020112_test
export START_TIME=2006020112
export FCST_TIME=03
export MODEL="WRF-RR NMM"


# Set up some constants

# Print run parameters
${ECHO}
${ECHO} "bucket.ksh started at `${DATE}`"
${ECHO}
${ECHO} "MOAD_DATAROOT = ${MOAD_DATAROOT}"
${ECHO} "     EXE_ROOT = ${EXE_ROOT}"

# Check to make sure the EXE_ROOT var was specified
if [ ! -d ${EXE_ROOT} ]; then
  ${ECHO} "ERROR: EXE_ROOT, '${EXE_ROOT}', does not exist"
  exit 1
fi

# Check to make sure the bucket executable exists
if [ ! -x ${BUCKET} ]; then
  ${ECHO} "ERROR: ${BUCKET} does not exist, or is not executable"
  exit 1
fi

# Check to make sure that the MOAD_DATAROOT exists
if [ ! -d ${MOAD_DATAROOT} ]; then
  ${ECHO} "ERROR: MOAD_DATAROOT, '${MOAD_DATAROOT}', does not exist"
  exit 1
fi

# If START_TIME is not defined, use the current time
if [ ! "${START_TIME}" ]; then
  START_TIME=`${DATE} +"%Y%m%d %H"`
else
  if [ `${ECHO} "${START_TIME}" | ${AWK} '/^[[:digit:]]{10}$/'` ]; then
    START_TIME=`${ECHO} "${START_TIME}" | ${SED} 's/\([[:digit:]]\{2\}\)$/ \1/'`
  elif [ ! "`${ECHO} "${START_TIME}" | ${AWK} '/^[[:digit:]]{8}[[:blank:]]{1}[[:digit:]]{2}$/'`" ]; then
    ${ECHO} "ERROR: start time, '${START_TIME}', is not in 'yyyymmddhh' or 'yyyymmdd hh' format"
    exit 1
  fi
  START_TIME=`${DATE} -d "${START_TIME}"`
fi

# Print out times
${ECHO} "   START TIME = "`${DATE} +%Y%m%d%H -d "${START_TIME}"`
${ECHO} "    FCST_TIME = ${FCST_TIME}"

# Set up the work directory and cd into it
workdir=${MOAD_DATAROOT}/bucketprd/${FCST_TIME}
${RM} -rf ${workdir}
${MKDIR} -p ${workdir}
cd ${workdir}

# Set the prefix for the input filename
if [ "${MODEL}" == "WRF-RR NMM" ]; then
  PREFIX=WRFTWO
  BUCKET=${EXE_ROOT}/WRFBUCKET_NMM.x
elif [ "${MODEL}" == "WRF-RR ARW" ]; then
  PREFIX=WRFTWO
  BUCKET=${EXE_ROOT}/WRFBUCKET_ARW.x
else
  ${ECHO} "Unsupported model: ${MODEL}"
  exit 1
fi

# Create the input file for bucket
${CAT} > input.card <<EOF
${MOAD_DATAROOT}/postprd
${PREFIX}
${MOAD_DATAROOT}/bucketprd
${FCST_TIME}
EOF

${BUCKET} < input.card
error=$?
if [ ${error} -ne 0 ]; then
  ${ECHO} "${BUCKET} crashed!  Exit status=${error}"
  exit ${error}
fi

# Check to make sure all output files were produced
if [ ! -s "${MOAD_DATAROOT}/bucketprd/WRFPCP${FCST_TIME}.tm00" ]; then
  ${ECHO} "wrfbucket crashed! WRFPCP${FCST_TIME}.tm00 is missing"
  exit 1
fi

# Copy the bucket file to /exchange
##if [ "${EXCHANGE_DIR}" ]; then
##  ${MKDIR} -p -m 2775 ${EXCHANGE_DIR}
##  ${MKDIR} -p ${EXCHANGE_DIR}
##  ${CP} ${MOAD_DATAROOT}/bucketprd/WRFPCP${FCST_TIME}.tm00 ${EXCHANGE_DIR}
##  error=$?
##  if [ ${error} -ne 0 ]; then
##    ${ECHO} "Copy to /exchange failed!  Exit status=${error}"
##    exit ${error}
##  fi

##  # Make sure bucket file made it to exchange
##  if [ ! -s "${EXCHANGE_DIR}/WRFPCP${FCST_TIME}.tm00" ]; then
##    ${ECHO} "Copy to /exchange failed! ${EXCHANGE_DIR}/WRFPCP${FCST_TIME}.tm00 is missing"
##    exit 1
##  fi
##fi

# Remove the workdir
${RM} -rf ${workdir}

${ECHO} "bucket.ksh completed at `${DATE}`"

